#include "stdafx.h"
#include "DataLoader.h"

DataLoader::DataLoader(std::string fn)
{
	fileName = fn;
	bufferIsShort = false;

	tsPacketCounter = 0;
	errorTsPacketCounter = 0;
	lostTsPacketCounter = 0;
	
	synchronized = false;
	patLoaded = false;
	pmtLoaded = false;
	nitActualLoaded = false;
	nitOtherLoaded = false;
	sdtActualLoaded = false;
	sdtOtherLoaded = false;
	batLoaded = false;
	eitActualPLoaded = false;
	eitOtherPLoaded = false;
	eitActualSLoaded = false;
	eitOtherSLoaded = false;
	rstLoaded = false;
	tdtLoaded = false;
	totLoaded = false;

	numberOfPat = 0;
	numberOfCat = 0;
	numberOfPmts = 0;
	numberOfTsdt = 0;
	numberOfNitActual = 0;
	numberOfNitOther = 0;
	numberOfSdtActual = 0;
	numberOfSdtOther = 0;
	numberOfBat = 0;
	numberOfEitActualP = 0;
	numberOfEitOtherP = 0;
	numberOfEitActualS = 0;
	numberOfEitOtherS = 0;
	numberOfCit = 0;
	numberOfRst = 0;
	numberOfTdt = 0;
	numberOfTot = 0;
	numberOfSt = 0;
	numberOfSyncNetwork = 0;
	numberOfRnt = 0;
	numberOfMetaData = 0;
	numberOfMeasurement = 0;
	numberOfDit = 0;
	numberOfSit = 0;

	numberOfPrograms = 0;
	tables = NULL;
}

DataLoader::~DataLoader(void)
{
	if(tables){delete tables;}
}

bool DataLoader::openFile(string fn){
	fr = fopen(fn.c_str(),"rb");
	if (!fr)
	{
		//chyba pri otevirani souboru
		return false;
	}

	return true;
}

void DataLoader::loadData(){
	// open file
	if(!openFile(fileName))
	{
		throw CannotOpenFileException("Cannot open input file");
	}

	if(tables){delete tables;}
	tables = new TablesContainer();
	
	//smycka hledani tabulek SI
	while (!patLoaded || !pmtLoaded || !nitActualLoaded || !nitOtherLoaded || !sdtActualLoaded || !sdtOtherLoaded || !batLoaded || !eitActualPLoaded || !eitOtherPLoaded || !eitActualSLoaded || !eitOtherSLoaded || !rstLoaded || !tdtLoaded || !totLoaded)
	{
		if (tsPacketCounter > 438830){break;}	//797873 - pocet paketu pro 40 Mbit/s	//438830 - pocet paketu pro 22 Mbit/s
		//synchronize data
		try 
		{
			synchronizePos();
		}
		catch(SynchronizationException syncex)
		{
			cerr << syncex.what() << endl;
			cout << "Synchronized exception. Return from loadDataFunction" << endl;
			fclose(fr);
			return;
		}

		//cte hlavicku ts vyclenuje chybne pakety a pakety kde nezacina PES
		try
		{
			//funkce error ts packet cte druhy byte v hlavicce TS paketu, pocita pakety OK i KO
			if (!errorTsPacket() && startPes())
			{
				if (readPID() <= 0x1F)
				{
					siTableIdentity();		//identifikuje tabulky SI a vola funkce pro jejich plneni
				}
				else
				{
					pmtsTableIdentity();	//identifikuje PID > 0x1F jestli to jsou PID PMT tabulek, vola funkci pro plneni PMT
				}
			}
			//zachyceni zbytku tabulek je volano az z funkci pro plneni tabulek
		}
		catch (ScrambledException scramex)
		{
			//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
			cerr << scramex.what() << endl;
			cout << "Find scrambled TS packet" << endl;
		}
		catch (AdaptionFieldException afex)
		{
			//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
			cerr << afex.what() << endl;
			cout << "Find TS packet with adaption field" << endl;
		}
		catch (BadTableIDException btidex)
		{
			cerr << btidex.what() << endl;
		}
	}
	fclose(fr);
}

int DataLoader::readByte()
{
	int chr;
	chr = getc(fr); //nalezeni prvniho bytu 47hex
	//fread(buff, 1, 1, fr);	
	if(feof(fr))
	{
		throw SynchronizationException("End of file when /synchronizing/ read data");
	}

	/*if (fseek(fr,1L,SEEK_CUR) == 0)
	{
		fseek(fr,-1L,SEEK_CUR);
	}
	else
	{
		bufferIsShort = true;
	}*/

	return chr;
}

void DataLoader::findNextByte47()
{
	int chr;
	int bytesRead = 0;
	do
	{
		chr = readByte();
		//cout << "Nacteny byte " << chr << endl;
		bytesRead++;
	}
	while (chr != 0x47 || bytesRead > 200);
	//ungetc(chr, fr);
}

void DataLoader::synchronizePos()
{
	//* Pocet pokusu hledani bytu 47
	int attempts = 10;
	//* Pozice bytu 47+1 v souboru
	fpos_t pos47;
	//* Nacitany znak
	int chr;
	while(attempts > 0)
	{
		//* Najdi dalsi byte 47
		findNextByte47();
		//* Pokracovav hledani dalsiho bytu 47
		bool tryNext3 = false;
		//* uloz aktualni pozici c souboru
		fgetpos(fr, &pos47);
		//* zkus najit byte 47 5x yasebou
		for (int i = 0; i < 5; i++)
		{
			//* posun na dalsi record
			fseek(fr, 187, SEEK_CUR);
			chr = readByte();
			if(chr != 0x47)
			{
				//* byte 47 nenalezen. Zkus najit dalsi byte 47
				tryNext3 = true;
				break;
			}
		}
		//* Vrat se zpatky za puvodni byte 47
		fsetpos(fr, &pos47);
		if(tryNext3)
		{
			//cout << "Synchronize NOT OK - dalsi pokus o nalezeni byty 47" << endl;
			attempts--;
		} else
		{
			//cout << "Synchronize OK" << endl;
			break;
		}
	}
	
	//ungetc(chr, fr);
}

bool DataLoader::errorTsPacket()
{
	tsPacketCounter++;
	byte2TsHead = readByte();
	if (byte2TsHead >= 128)
	{
		errorTsPacketCounter++;
		return true;
	}
	return false;
}

bool DataLoader::startPes()
{
	if (byte2TsHead >= 64)
	{
		return true;
	}
	return false;
}

int DataLoader::readPID()
{
	PID = ((binaryShift(byte2TsHead,4,8))*0x100)+readByte();
	return PID;
}

int DataLoader::binaryShift(int byte, int msb, int lsb)
{
	//Funkce vybere s binarni posloupnosti byte cast ohranicenou msb a lsb a vrati hodnotu takto vybraneho cisla, msb a lsb jsou cislonavy od 1
	int msb_pom = msb-1;
	while (msb_pom != 0)
	{
		byte = byte << 1;
		if (byte >= 256)
		{
			byte = byte-256;	
		}
		msb_pom--;
	}
	byte = byte >> (msb-1);
	byte = byte >> (8-lsb);

	return byte;
}

int DataLoader::continuityCounter()
{
	int byte4TsHead = readByte();
	if (binaryShift(byte4TsHead,1,2) > 0)
	{
		cerr << "data jsou scramblovany";
		throw ScrambledException("Scrambling data, not decoded packet");
	}
	if (binaryShift(byte4TsHead,3,4) != 1)
	{
		cout << "adaptacni pole";
		throw AdaptionFieldException("Found adaption field, error because SI tables can not contain adaption field, not decoded packet");
	}
	continuityCount = binaryShift(byte4TsHead,5,8);
	return continuityCount;
}

int DataLoader::readTableID()
{
	if (readByte() != 0)
	{
		cout << "Warning: First byte in ts packet is not a zero" << endl;
	}
	return readByte();
}

void DataLoader::questionAtEndTsPacket()
{
	if (counterByte == 188)
	{
		//check continouns data
		if (readByte() != 0x47)
		{
			throw SynchronizationException("Chyba synchronizace pri nacitani tabulky delsi nez jeden TS packet");
		}

		//funkce error ts packet cte druhy byte v hlavicce TS paketu, pocita pakety OK i KO
		if (!errorTsPacket())
		{
			previousPID = PID;
			if (continuityCount == 0xF)
			{
				previousContinuityCount = -1;
			}
			else
			{
				previousContinuityCount = continuityCount;
			}
			int watchdog = 0;
			fgetpos(fr, &synchronizedPosition);
			while (watchdog != 30)
			{
				if (startPes() == false && readPID() == previousPID && continuityCounter() == previousContinuityCount+1)
				{
					counterByte = 4;
					return;
				}
				synchronizePos();
				if (errorTsPacket())
				{
					//lostTsPacketCounter++;
					throw LostTsPacketException("Ztrata TS paketu");
				}
				watchdog++;
			}
		}
		lostTsPacketCounter++;
		throw LostTsPacketException("Ztrata TS paketu");
	}
}
