#ifndef DATALOADER_H
#define DATALOADER_H

#include <iostream>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include <vector>
#include <stdexcept>

#include "TableContainer.h"
#include "exceptions.h"

using namespace std;

enum DescriptorName
{
	DN_NETWORK_NAME	= 0x40,
	DN_SERVICE_LIST = 0x41,
	DN_BOUQUET_NAME = 0x47,
	DN_SERVICE = 0x48,
	DN_SHORT_EVENT = 0x4D,
	DN_EXTENDED_EVENT = 0x4E,
	DN_COMPONENT = 0x50,
	DN_CONTENT = 0x54,
	DN_TERRESTRIAL_DELIVERY_SYSTEM = 0x5A,
	DN_FREQUENCY_LIST = 0x62,
	DN_APPLICATION_SIGNALLING = 0x6F,
	DN_TELETEXT = 0x56,
	DN_LOCAL_TIME_OFFSET = 0x58,
};

enum TableName
{
	TN_PAT = 0x00,
	TN_CAT = 0x01,
	TN_PMT = 0x02,
	TN_TSDT = 0x03,
	TN_NIT_ACTUAL = 0x40,
	TN_NIT_OTHER = 0x41,
	TN_SDT_ACTUAL = 0x42,
	TN_SDT_OTHER = 0x46,
	TN_BAT = 0x4A,
	TN_EIT_ACTUAL_P = 0x4E,
	TN_EIT_OTHER_P = 0x4F,
	TN_TDT = 0x70,
	TN_RST = 0x71,
	TN_ST = 0x72,
	TN_TOT = 0x73,
};

class DataLoader
{
private:
	int numberOfPrograms;
	int numberOfProgramsOther;
	vector<int> programNumbers;
	vector<int> programNumbersWithoutNit;
	vector<int> pmtPIDs;
	vector<int> pmtPIDsWithoutNit;
	bool bufferIsShort;
	FILE* fr;
	fpos_t synchronizedPosition;
	fpos_t actualPosition;

	void findNextByte47();
	int readByte();

	int infoLength;

	TablesContainer* tables;
	void synchronizePos();
	
	//unsigned long synchronizePosition(std::string fileName);
	
	void loadPat();
	void loadPmt();

	void loadNitActual();
	void loadNitOther();
	
	void loadSdtActual();
	void loadSdtOther();

	void loadBat();

	//EIT
	Eit oneSectionEit;
	void loadEitBody();

	//eit actual present
	void loadEitActualP();
	void locateAndSaveSectionEitActualP();

	//eit other present
	void loadEitOtherP();
	void locateAndSaveSectionEitOtherP();

	//eit actual schedule
	void loadEitActualS();
	void locateAndSaveSectionEitActualS();
	bool completeEitActualS();

	//eit other schedule
	void loadEitOtherS();
	void locateAndSaveSectionEitOtherS();
	bool completeEitOtherS();

	//RST
	void loadRst();
	void loadTdt();
	void loadTot();

	int tsPacketCounter;
	int errorTsPacketCounter;
	int byte2TsHead;
	bool errorTsPacket();
	bool startPes();

	int binaryShift(int, int, int);

	int PID;
	int readPID();

	int continuityCount;
	int continuityCounter();

	int tableID;
	void siTableIdentity();
	void pmtsTableIdentity();
	int readTableID();

	vector<Descriptor> tableDescriptors;
	vector<Descriptor> loopDescriptors;
	void descriptorIdentity(bool descriptorLocationInLoop);//pokud bude funkce volana z tabulky tak descriptorLocationInLoop = false, pokud bude funkce volana ze smyek uitench dat tak descriptorLocationInLoop = true
	//void loopDescriptorIdentity();

	int counterByte;
	int numberOfNotLoadedByte;

	int numberOfNotLoadedEitActualPService;
	int numberOfNotLoadedEitActualSService;
	int numberOfNotLoadedEitOtherPService;
	int numberOfNotLoadedEitOtherSService;

	void questionAtEndTsPacket();

	int previousPID;
	int previousContinuityCount;
	int lostTsPacketCounter;

	Descriptor oneDescriptor;
	
	void loadNetworkNameDescriptor();
	void loadServiceListDescriptor();
	void loadBouquetNameDesctiptor();
	void loadServiceDescriptor();
	void loadShortEventDescriptor();
	void loadExtendedEventDescriptor();
	void loadComponentDescriptor();
	void loadContentDescriptor();
	void loadTeletextDescriptor();
	void loadTerrestrialDeliverySystemDescriptor();
	void loadFrequencyListDescriptor();
	void loadApplicationSignallingDescriptor();
	void loadLocalTimeOffsetDescriptor();
	void loadRawDataDescriptor();

	int numberOfPat;
	int numberOfCat;
	int numberOfPmts;
	int numberOfTsdt;
	int numberOfNitActual;
	int numberOfNitOther;
	int numberOfSdtActual;
	int numberOfSdtOther;
	int numberOfBat;
	int numberOfEitActualP;
	int numberOfEitOtherP;
	int numberOfEitActualS;
	int numberOfEitOtherS;
	int numberOfCit;
	int numberOfRst;
	int numberOfTdt;
	int numberOfTot;
	int numberOfSt;
	int numberOfSyncNetwork;
	int numberOfRnt;
	int numberOfMetaData;
	int numberOfMeasurement;
	int numberOfDit;
	int numberOfSit;

	std::string fileName;
	
public:
	DataLoader(std::string fn);
	~DataLoader();
	
	bool openFile(string fn);
	
	void loadData();
	
	bool synchronized;
	bool patLoaded;
	bool pmtLoaded;
	bool nitActualLoaded;
	bool nitOtherLoaded;
	bool sdtActualLoaded;
	bool sdtOtherLoaded;
	bool batLoaded;
	bool eitActualPLoaded;
	bool eitOtherPLoaded;
	bool eitActualSLoaded;
	bool eitOtherSLoaded;
	bool rstLoaded;
	bool tdtLoaded;
	bool totLoaded;

	/*bool pmtSearching;
	bool nitSearching;
	bool sdtSearching;
	bool batSearching;
	bool eitPSearching;
	bool eitSSearching;
	bool tdtSearching;*/
	
	TablesContainer* getData(){return tables;}
	//TablesContainer getData(){return tables;}

	int getNumberOfTsPacket(){return tsPacketCounter;}
	int getNumberOfErrorTsPacket(){return errorTsPacketCounter;}
	int getNumberOfLostTsPacket(){return lostTsPacketCounter;}

	int getNumberOfPat(){return numberOfPat;}
	int getNumberOfPmts(){return numberOfPmts;}
	int getNumberOfNitActual(){return numberOfNitActual;}
	int getNumberOfNitOther(){return numberOfNitOther;}
	int getNumberOfSdtActual(){return numberOfSdtActual;}
	int getNumberOfSdtOther(){return numberOfSdtOther;}
	int getNumberOfBat(){return numberOfBat;}
	int getNumberOfEitActualP(){return numberOfEitActualP;}
	int getNumberOfEitActualS(){return numberOfEitActualS;}
	int getNumberOfEitOtherP(){return numberOfEitOtherP;}
	int getNumberOfEitOtherS(){return numberOfEitOtherS;}
	int getNumberOfTdt(){return numberOfTdt;}
	int getNumberOfTot(){return numberOfTot;}

	int getNumberOfCat(){return numberOfCat;}
	int getNumberOfTsdt(){return numberOfTsdt;}
	int getNumberOfCit(){return numberOfCit;}
	int getNumberOfRst(){return numberOfRst;}
	int getNumberOfSt(){return numberOfSt;}
	int getNumberOfSyncNetwork(){return numberOfSyncNetwork;}
	int getNumberOfRnt(){return numberOfRnt;}
	int getNumberOfMetaData(){return numberOfMetaData;}
	int getNumberOfMeasurement(){return numberOfMeasurement;}
	int getNumberOfDit(){return numberOfDit;}
	int getNumberOfSit(){return numberOfSit;}
};

#endif
