#include "stdafx.h"
#include "DataLoader.h"
#include "TableContainer.h"
#include "exceptions.h"

/******************************************************PAT TABLE*******************************************************/
void DataLoader::loadPat()
{
	if (patLoaded){return;}													//overeni, zda tabulka jeste neni nactena

	counterByte = 4+2;														//nastaveni pocitadla prectenzch bajtu (4 bajty hlavicky paketu TS + 2 bajty ID tabulky
	tables->PAT.setTableID(tableID);										//ulozeni hodnoty ID tabulky
	try																		//blok try pro osetreni vyjimek behem nacitani
	{
		int byteBuff = readByte();											//metoda readByte() cte jednotlive bajty, hodnota je ulozena do pomocne promenne
		counterByte++;														//zvysena hodnota prectenych bajtu - vzdy po volani readByte()
		tables->PAT.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));	//nacitani hlavicky tabulky PAT, metoda binaryShift() vraci cislo po bitovem posuvu, posuv je zavisly na vstupnich parametrech
		tables->PAT.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->PAT.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->PAT.getSectionLength() - 4;			//numberOfNotLoadedByte je postupne odecitana, jakmile je nula je tabulka nactena, -4 je zde protoze neni nacitan cyklicky soucet na konci CRC

		if (tables->PAT.getSectionSyntaxIndicator())						//rozhodovani zda se jedna o dlouho nebo kratkou tabulku
		{
			tables->PAT.setTransportStreamID(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			tables->PAT.setVersionNumb(binaryShift(byteBuff,3,7));
			tables->PAT.setCurrentIndicator(binaryShift(byteBuff,8,8));
			tables->PAT.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			tables->PAT.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;										//konec nacitani hlavicky tabulky PAT
		}
		if (tables->PAT.getSectionLength() > 176)							//pokut jsou data v tabulce delsi nez jeden paket TS - v pripade PAT vyhozena vyjimka
		{
			//pat je kratka takze vyhodi vyjimku
			throw TooLongTableException("Table PAT contain is too long data");
		}

		while (numberOfNotLoadedByte != 0)									//smycka nacitani tela tabulky, testovan numberOfNotLoadedByte na nulu
		{
			programNumbers.push_back(readByte()*0x100+readByte());			//nacitani cisel programu
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			pmtPIDs.push_back(binaryShift(readByte(),4,8)*0x100+readByte());	//nacitani PID PMT tabulek
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			if (counterByte > 188)											//kontrola zda nedoslo ke cteni mimo paket TS
			{
				throw ReadDataOutsideTsPacketException("Read data outside TS packet when loading PAT");
			}
		}
		if (numberOfNotLoadedByte < 0)										//kontrola zda nedoslo k behem smycky while k chybe a numberOfNotLoadedByte jiz neni zaporny
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading PAT");
		}
	}
	catch(TooLongTableException tLTex)										//odchytavani vyjimek
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(ReadDataOutsideTsPacketException rdoTSpex)
	{
		cerr << rdoTSpex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	//pocet programu - bez nit
	for (unsigned int m=0;m<programNumbers.size();m++)						//zjistovani poctu programu a filtrovani cisla programu 0 a PID 0x10 - NIT tabulka
	{
		if (programNumbers.at(m) != 0)
		{
			numberOfPrograms++;
			programNumbersWithoutNit.push_back(programNumbers.at(m));
			pmtPIDsWithoutNit.push_back(pmtPIDs.at(m));
		}
	}
	numberOfNotLoadedEitActualPService = numberOfPrograms;					//nastaveni poctu programu pro nacitani jinych tabulek
	numberOfNotLoadedEitActualSService = numberOfPrograms;

	tables->PAT.setProgramNumbers(programNumbers);							//predani nactenych informaci do datoveho kontejneru
	tables->PAT.setProgramNumbersWithoutNit(programNumbersWithoutNit);
	tables->PAT.setPmtPIDsWithoutNit(pmtPIDsWithoutNit);
	tables->PAT.setPmtPIDs(pmtPIDs);
	
	patLoaded = true;														//nastaveni prizanku korektniho nacteni PAT tabulky
}

/******************************************************PMT TABLES*******************************************************/
void DataLoader::loadPmt()
{
	if (pmtLoaded){return;}

	Pmt pmt;
	counterByte = 4+2;
	StreamLoop oneStreamLoop;
	vector<StreamLoop> streamsLoopOneProgram;
		
	pmt.setTableID(tableID);
	try
	{
		#pragma region Header

		int byteBuff = readByte();
		counterByte++;

		pmt.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		pmt.setPrivateFlag(binaryShift(byteBuff,2,2));
		pmt.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = pmt.getSectionLength() - 4; //4 kvuli odecteni CRC

		if (pmt.getSectionSyntaxIndicator())
		{
			pmt.setProgramNumber(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			pmt.setVersionNumb(binaryShift(byteBuff,3,7));
			pmt.setCurrentIndicator(binaryShift(byteBuff,8,8));
			pmt.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			pmt.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
		}
		if (pmt.getSectionLength() > 176)
		{
			//bude plneni pres buffery ale pmt je kratka takze vyhodi vyjimku
			throw TooLongTableException("Table PMT contain is too long data");
		}

		//PID
		pmt.setPmtPID(PID);

		//PCR PID
		pmt.setPcrPID(binaryShift(readByte(),4,8)*0x100+readByte());
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//pmt descriptors length (infoLength)
		infoLength = binaryShift(readByte(),5,8)*0x100+readByte();
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//pmt descriptors
		while (infoLength != 0)
		{
			descriptorIdentity(false);
			if (counterByte > 188){throw ReadDataOutsideTsPacketException("Read data outside TS packet when loading PMT");}
		}
		if (numberOfNotLoadedByte < 0)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading PMT");
		}
		pmt.setPmtDescriptors(tableDescriptors);
		tableDescriptors.resize(0);

		#pragma endregion Header PMT

		//Stream Loop
		#pragma region Body

		while (numberOfNotLoadedByte != 0)
		{
			//stream type
			oneStreamLoop.setStreamType(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			//elementary PID
			oneStreamLoop.setElementaryPID(binaryShift(readByte(),4,8)*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			//ES info length
			infoLength = binaryShift(readByte(),5,8)*0x100+readByte();
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			//Descriptors
			while (infoLength != 0)
			{
				descriptorIdentity(true);
				if (counterByte > 188){throw ReadDataOutsideTsPacketException("Read data outside TS packet when loading PMT");}
			}
			oneStreamLoop.setStreamLoopDescriptors(loopDescriptors);
			loopDescriptors.resize(0);
			streamsLoopOneProgram.push_back(oneStreamLoop);	//predani jedne streamove smycky do vektoru vsech streamovych smycek jednoho programu
			if (numberOfNotLoadedByte < 0)
			{
				throw BadInformationAboutLengthDataException("Bad information about length data when loading PMT");
			}
		}
		#pragma endregion Stream Loops
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(ReadDataOutsideTsPacketException rdoTSpex)
	{
		cerr << rdoTSpex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	pmt.setStreamLoops(streamsLoopOneProgram);	//predani vektoru se vsemi smyckami jednoho programu do TableContaineru - vector<StreamLoops>
	tables->PMT.push_back(pmt);					//predani pmt do TableContaineru
	if (numberOfPrograms == tables->PMT.size())
	{
		pmtLoaded = true;
	}
}

/******************************************************NIT TABLES*******************************************************/
void DataLoader::loadNitActual()
{
	if (nitActualLoaded){return;}

	counterByte = 4+2;
	TransportStreamLoop oneTransportStreamLoop;
	vector<TransportStreamLoop> transportStreamsLoops;
	tables->NIT_ACTUAL.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->NIT_ACTUAL.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->NIT_ACTUAL.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->NIT_ACTUAL.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->NIT_ACTUAL.getSectionLength()-4;

		if (tables->NIT_ACTUAL.getSectionSyntaxIndicator())
		{
			tables->NIT_ACTUAL.setNetworkID(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			tables->NIT_ACTUAL.setVersionNumb(binaryShift(byteBuff,3,7));
			tables->NIT_ACTUAL.setCurrentIndicator(binaryShift(byteBuff,8,8));
			tables->NIT_ACTUAL.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			tables->NIT_ACTUAL.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
		}
		
		//nit descriptors length (infoLength)
		infoLength = binaryShift(readByte(),5,8)*0x100+readByte();
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;
		
		//nit descriptors
		while (infoLength != 0)
		{
			descriptorIdentity(false);
		}
		if (numberOfNotLoadedByte < 0)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading NIT_ACTUAL");
		}
		tables->NIT_ACTUAL.setNitDescriptors(tableDescriptors);
		tableDescriptors.resize(0);

		//transport stream loop length (infoLength)
		byteBuff = readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();

		//overeni sparavne delky nacitanych dat
		if (infoLength != numberOfNotLoadedByte) //4 protoze CRC je take zapocitano do delky
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading NIT_ACTUAL");
		}

		//trasport stream loop
		while (numberOfNotLoadedByte != 0) //4 protoze CRC je take zapocitano do delky
		{
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneTransportStreamLoop.setTransportStreamID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneTransportStreamLoop.setOriginalNetworkID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//trasport descriptors length
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//Descriptors
			while (infoLength != 0)
			{
				descriptorIdentity(true);
			}
			oneTransportStreamLoop.setTransportStreamLoopDescriptors(loopDescriptors);
			loopDescriptors.resize(0);
			transportStreamsLoops.push_back(oneTransportStreamLoop);
			if (numberOfNotLoadedByte < 0)
			{
				throw BadInformationAboutLengthDataException("Bad information about length data when loading NIT_ACTUAL");
			}
		}
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	tables->NIT_ACTUAL.setTransportStreamLoops(transportStreamsLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<transportStreamsLoops>
	nitActualLoaded = true;
}

void DataLoader::loadNitOther()
{
	if (nitOtherLoaded){return;}

	counterByte = 4+2;
	TransportStreamLoop oneTransportStreamLoop;
	vector<TransportStreamLoop> transportStreamsLoops;
	tables->NIT_OTHER.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->NIT_OTHER.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->NIT_OTHER.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->NIT_OTHER.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->NIT_OTHER.getSectionLength()-4;

		if (tables->NIT_OTHER.getSectionSyntaxIndicator())
		{
			tables->NIT_OTHER.setNetworkID(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			tables->NIT_OTHER.setVersionNumb(binaryShift(byteBuff,3,7));
			tables->NIT_OTHER.setCurrentIndicator(binaryShift(byteBuff,8,8));
			tables->NIT_OTHER.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			tables->NIT_OTHER.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
		}

		//nit descriptors length (infoLength)
		infoLength = binaryShift(readByte(),5,8)*0x100+readByte();
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//nit descriptors
		while (infoLength != 0)
		{
			descriptorIdentity(false);
		}
		if (numberOfNotLoadedByte < 0)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading NIT_OTHER");
		}
		tables->NIT_OTHER.setNitDescriptors(tableDescriptors);
		tableDescriptors.resize(0);

		//transport stream loop length (infoLength)
		byteBuff = readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();

		//overeni sparavne delky nacitanych dat
		if (infoLength != numberOfNotLoadedByte) //4 protoze CRC je take zapocitano do delky
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading NIT_OTHER");
		}

		//trasport stream loop
		while (numberOfNotLoadedByte != 0) //4 protoze CRC je take zapocitano do delky
		{
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneTransportStreamLoop.setTransportStreamID(readByte()*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneTransportStreamLoop.setOriginalNetworkID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//trasport descriptors length
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//Descriptors
			while (infoLength != 0)
			{
				descriptorIdentity(true);
			}
			oneTransportStreamLoop.setTransportStreamLoopDescriptors(loopDescriptors);
			loopDescriptors.resize(0);
			transportStreamsLoops.push_back(oneTransportStreamLoop);
			if (numberOfNotLoadedByte < 0)
			{
				throw BadInformationAboutLengthDataException("Bad information about length data when loading NIT_OTHER");
			}
		}
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	tables->NIT_OTHER.setTransportStreamLoops(transportStreamsLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<transportStreamsLoops>
	nitOtherLoaded = true;
}

/******************************************************SDT TABLES*******************************************************/
void DataLoader::loadSdtActual()
{
	if (sdtActualLoaded){return;}

	counterByte = 4+2;
	ServiceLoop oneServiceLoop;
	vector<ServiceLoop> serviceLoops;
	tables->SDT_ACTUAL.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->SDT_ACTUAL.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->SDT_ACTUAL.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->SDT_ACTUAL.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->SDT_ACTUAL.getSectionLength()-4;

		if (tables->SDT_ACTUAL.getSectionSyntaxIndicator())
		{
			tables->SDT_ACTUAL.setTransportStreamID(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			tables->SDT_ACTUAL.setVersionNumb(binaryShift(byteBuff,3,7));
			tables->SDT_ACTUAL.setCurrentIndicator(binaryShift(byteBuff,8,8));
			tables->SDT_ACTUAL.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			tables->SDT_ACTUAL.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
		}
	
		//original network ID
		tables->SDT_ACTUAL.setOriginalNetworkID(readByte()*0x100+readByte());
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//reserved byte
		readByte();
		counterByte++;
		numberOfNotLoadedByte--;

		while (numberOfNotLoadedByte != 0)
		{
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneServiceLoop.setServiceID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneServiceLoop.setEitScheduleFlag(binaryShift(byteBuff,7,7));
			oneServiceLoop.setEitPresentFollowingFlag(binaryShift(byteBuff,8,8));

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneServiceLoop.setRunningStatus(binaryShift(byteBuff,1,3));
			oneServiceLoop.setFreeOrCaMode(binaryShift(byteBuff,4,4));

			//service descriptors length
			infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//Descriptors
			while (infoLength != 0)
			{
				descriptorIdentity(true);
			}
			fgetpos(fr, &synchronizedPosition);
			oneServiceLoop.setServiceLoopDescriptors(loopDescriptors);
			loopDescriptors.resize(0);
			serviceLoops.push_back(oneServiceLoop);
			if (numberOfNotLoadedByte < 0)
			{
				throw BadInformationAboutLengthDataException("Bad information about length data when loading SDT_ACTUAL");
			}
		}
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	tables->SDT_ACTUAL.setServiceLoops(serviceLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<serviceLoops>
	sdtActualLoaded = true;
}

void DataLoader::loadSdtOther()
{
	if (sdtOtherLoaded){return;}

	counterByte = 4+2;
	ServiceLoop oneServiceLoop;
	vector<ServiceLoop> serviceLoops;
	tables->SDT_OTHER.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->SDT_OTHER.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->SDT_OTHER.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->SDT_OTHER.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->SDT_OTHER.getSectionLength()-4;

		if (tables->SDT_OTHER.getSectionSyntaxIndicator())
		{
			tables->SDT_OTHER.setTransportStreamID(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			tables->SDT_OTHER.setVersionNumb(binaryShift(byteBuff,3,7));
			tables->SDT_OTHER.setCurrentIndicator(binaryShift(byteBuff,8,8));
			tables->SDT_OTHER.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			tables->SDT_OTHER.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
		}

		//original network ID
		tables->SDT_OTHER.setOriginalNetworkID(readByte()*0x100+readByte());
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//reserved byte
		readByte();
		counterByte++;
		numberOfNotLoadedByte--;

		while (numberOfNotLoadedByte != 0)
		{
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneServiceLoop.setServiceID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneServiceLoop.setEitScheduleFlag(binaryShift(byteBuff,7,7));
			oneServiceLoop.setEitPresentFollowingFlag(binaryShift(byteBuff,8,8));

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneServiceLoop.setRunningStatus(binaryShift(byteBuff,1,3));
			oneServiceLoop.setFreeOrCaMode(binaryShift(byteBuff,4,4));

			//service descriptors length
			infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//Descriptors
			while (infoLength != 0)
			{
				descriptorIdentity(true);
			}
			fgetpos(fr, &synchronizedPosition);
			oneServiceLoop.setServiceLoopDescriptors(loopDescriptors);
			loopDescriptors.resize(0);
			serviceLoops.push_back(oneServiceLoop);
			if (numberOfNotLoadedByte < 0)
			{
				throw BadInformationAboutLengthDataException("Bad information about length data when loading SDT_OTHER");
			}
		}
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	tables->SDT_OTHER.setServiceLoops(serviceLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<serviceLoops>

	//pocet programu other pro plneni EIT tabulek
	numberOfNotLoadedEitOtherPService = tables->SDT_OTHER.getServiceLoops().size();
	numberOfNotLoadedEitOtherSService = numberOfNotLoadedEitOtherPService;
	sdtOtherLoaded = true;
}

/******************************************************BAT TABLE*******************************************************/
void DataLoader::loadBat()
{
	if (batLoaded){return;}

	counterByte = 4+2;
	TransportStreamLoop oneTransportStreamLoop;
	vector<TransportStreamLoop> transportStreamsLoops;
	tables->BAT.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->BAT.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->BAT.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->BAT.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->BAT.getSectionLength()-4;

		if (tables->BAT.getSectionSyntaxIndicator())
		{
			tables->BAT.setBouquetID(readByte()*0x100+readByte());
			counterByte = counterByte+2;
			numberOfNotLoadedByte = numberOfNotLoadedByte-2;
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			tables->BAT.setVersionNumb(binaryShift(byteBuff,3,7));
			tables->BAT.setCurrentIndicator(binaryShift(byteBuff,8,8));
			tables->BAT.setSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			tables->BAT.setLastSectionNumb(readByte());
			counterByte++;
			numberOfNotLoadedByte--;
		}

		//bat descriptors length (infoLength)
		infoLength = binaryShift(readByte(),5,8)*0x100+readByte();
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//bat descriptors
		while (infoLength != 0)
		{
			descriptorIdentity(false);
		}
		tables->BAT.setNitDescriptors(tableDescriptors);
		tableDescriptors.resize(0);

		//transport stream loop length (infoLength)
		byteBuff = readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();

		//overeni sparavne delky nacitanych dat
		if (infoLength != numberOfNotLoadedByte)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading BAT");
		}

		//trasport stream loop
		while (numberOfNotLoadedByte != 0)
		{
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneTransportStreamLoop.setTransportStreamID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			oneTransportStreamLoop.setOriginalNetworkID(byteBuff*0x100+readByte());
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//trasport descriptors length
			byteBuff = readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();
			infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
			counterByte++;
			numberOfNotLoadedByte--;
			questionAtEndTsPacket();

			//Descriptors
			while (infoLength != 0)
			{
				descriptorIdentity(true);
			}
			oneTransportStreamLoop.setTransportStreamLoopDescriptors(loopDescriptors);
			loopDescriptors.resize(0);
			transportStreamsLoops.push_back(oneTransportStreamLoop);
			if (numberOfNotLoadedByte < 0)
			{
				throw BadInformationAboutLengthDataException("Bad information about length data when loading BAT");
			}
		}
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
		return;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
		return;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	tables->BAT.setTransportStreamLoops(transportStreamsLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<transportStreamsLoops>
	batLoaded = true;
}

/******************************************************EIT TABLES*******************************************************/
void DataLoader::loadEitBody()
{
	EventLoop oneEventLoop;
	vector<EventLoop> eventLoops;

	oneSectionEit.setTableID(tableID);
	int byteBuff = readByte();
	counterByte++;
	oneSectionEit.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
	if (!oneSectionEit.getSectionSyntaxIndicator())
	{
		throw BadInformationAboutLengthDataException("Bad section syntax indicator when loading EIT");
	}
	oneSectionEit.setPrivateFlag(binaryShift(byteBuff,2,2));
	oneSectionEit.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
	counterByte++;
	numberOfNotLoadedByte = oneSectionEit.getSectionLength()-4;
	oneSectionEit.setServiceID(readByte()*0x100+readByte());
	counterByte = counterByte+2;
	numberOfNotLoadedByte = numberOfNotLoadedByte-2;
	byteBuff = readByte();
	counterByte++;
	numberOfNotLoadedByte--;
	oneSectionEit.setVersionNumb(binaryShift(byteBuff,3,7));
	oneSectionEit.setCurrentIndicator(binaryShift(byteBuff,8,8));
	oneSectionEit.setSectionNumb(readByte());
	counterByte++;
	numberOfNotLoadedByte--;
	oneSectionEit.setLastSectionNumb(readByte());
	counterByte++;
	numberOfNotLoadedByte--;
	//trasport stream ID
	oneSectionEit.setTransportStreamID(readByte()*0x100+readByte());
	counterByte = counterByte+2;
	numberOfNotLoadedByte = numberOfNotLoadedByte-2;
	//original network ID
	oneSectionEit.setOriginalNetworkID(readByte()*0x100+readByte());
	counterByte = counterByte+2;
	numberOfNotLoadedByte = numberOfNotLoadedByte-2;
	//segment last section number
	oneSectionEit.setSegmentLastSectionNumber(readByte());
	counterByte++;
	numberOfNotLoadedByte--;
	//last table ID
	oneSectionEit.setLastTableID(readByte());
	counterByte++;
	numberOfNotLoadedByte--;
	while (numberOfNotLoadedByte != 0)
	{
		//event ID
		int byteBuff = readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		oneEventLoop.setEventID(byteBuff*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//MJD
		byteBuff = readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		int MJD = byteBuff*0x100+readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//start day
		int Yi = int((MJD-15078.2)/365.25);
		int Mi = int((MJD-14956.1-int(Yi*365.25))/30.6001);
		oneEventLoop.setStartDay(MJD-14956-int(Yi*365.25)-int(Mi*30.6001));
		//start month
		int K=0;
		if((Mi == 14)||(Mi == 15)){K=1;}
		oneEventLoop.setStartMonth(Mi-1-K*12);
		//start year
		int y;
		if((Yi+K-100) >= 0){y = 2000 + (Yi+K-100);}
		else{y = 1900 + (Yi+K-100);}
		oneEventLoop.setStartYear(y);
		//start day of week
		oneEventLoop.setStartDayOfWeek(((MJD+2)%7)+1);
		//start hour
		oneEventLoop.setStartHour(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//start minute
		oneEventLoop.setStartMinute(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//start second
		oneEventLoop.setStartSecond(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//duration hour
		oneEventLoop.setDurationHour(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//duration minute
		oneEventLoop.setDurationMinute(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//duration second
		oneEventLoop.setDurationSecond(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//running status
		byteBuff = readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		oneEventLoop.setRunningStatus(binaryShift(byteBuff,1,3));
		//free/CA mode
		oneEventLoop.setFreeOrCaMode(binaryShift(byteBuff,4,4));
		//descriptors loop length
		infoLength = binaryShift(byteBuff,5,8)*0x100+readByte();
		counterByte++;
		numberOfNotLoadedByte--;
		questionAtEndTsPacket();
		//Descriptors
		while (infoLength != 0)
		{
			descriptorIdentity(true);
		}
		oneEventLoop.setEventLoopDescriptors(loopDescriptors);
		loopDescriptors.resize(0);
		eventLoops.push_back(oneEventLoop);
		if (numberOfNotLoadedByte < 0)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading EIT");
		}
	}
	oneSectionEit.setEventLoops(eventLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<serviceLoops>
}

//eit actual present
void DataLoader::loadEitActualP()
{
	if (eitActualPLoaded){return;}
	if (!patLoaded){return;}
	counterByte = 4+2;
	try
	{
		loadEitBody();
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}

	locateAndSaveSectionEitActualP();
}

void DataLoader::locateAndSaveSectionEitActualP()
{
	vector<Eit> oneProgramEitActual;

	if (tables->EIT_ACTUAL_P.size() == 0)
	{
		oneProgramEitActual.push_back(oneSectionEit);
		tables->EIT_ACTUAL_P.push_back(oneProgramEitActual);
		return;
	}

	int numberOfLoadedProgramsEit = tables->EIT_ACTUAL_P.size();
	for (int m=0;m<numberOfLoadedProgramsEit;m++)
	{
		if (tables->EIT_ACTUAL_P.at(m).at(0).getServiceID() == oneSectionEit.getServiceID())
		{
			if (tables->EIT_ACTUAL_P.at(m).size() != 0)
			{
				int numberOfSectionInOneService = tables->EIT_ACTUAL_P.at(m).size();
				for (int n=0;n<numberOfSectionInOneService;n++)
				{
					if (tables->EIT_ACTUAL_P.at(m).size() == oneSectionEit.getLastSectionNumber()+1)
					{
						return;
					}
					if (tables->EIT_ACTUAL_P.at(m).at(n).getSectionNumber() != oneSectionEit.getSectionNumber())
					{
						tables->EIT_ACTUAL_P.at(m).push_back(oneSectionEit);
						if (tables->EIT_ACTUAL_P.at(m).size() == oneSectionEit.getLastSectionNumber()+1)
						{
							numberOfNotLoadedEitActualPService--;
							if (numberOfNotLoadedEitActualPService == 0)
							{
								eitActualPLoaded = true;
							}
						}
					}
				}
				return;
			}
			else
			{
				tables->EIT_ACTUAL_P.at(m).push_back(oneSectionEit);
				if (tables->EIT_ACTUAL_P.at(m).size() == oneSectionEit.getLastSectionNumber()+1)
				{
					numberOfNotLoadedEitActualPService--;
					if (numberOfNotLoadedEitActualPService == 0)
					{
						eitActualPLoaded = true;
					}
				}
				return;
			}
		}
	}

	oneProgramEitActual.push_back(oneSectionEit);
	tables->EIT_ACTUAL_P.push_back(oneProgramEitActual);
	if (tables->EIT_ACTUAL_P.back().size() == oneSectionEit.getLastSectionNumber()+1)
	{
		numberOfNotLoadedEitActualPService--;
		if (numberOfNotLoadedEitActualPService == 0)
		{
			eitActualPLoaded = true;
		}
	}
}

//void DataLoader::completeEitActualP(){} - zatim delano v locateAndSaveSectionEitActualP

//eit other present
void DataLoader::loadEitOtherP()
{
	if (eitOtherPLoaded){return;}
	if (!sdtOtherLoaded){return;}
	counterByte = 4+2;
	try
	{
		loadEitBody();
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	locateAndSaveSectionEitOtherP();
}

void DataLoader::locateAndSaveSectionEitOtherP()
{
	numberOfProgramsOther = tables->SDT_OTHER.getServiceLoops().size();
	vector<Eit> oneProgramEitOther;

	if (tables->EIT_OTHER_P.size() == 0)
	{
		oneProgramEitOther.push_back(oneSectionEit);
		tables->EIT_OTHER_P.push_back(oneProgramEitOther);
		return;
	}

	int numberOfLoadedProgramsEit = tables->EIT_OTHER_P.size();
	for (int m=0;m<numberOfLoadedProgramsEit;m++)
	{
		if (tables->EIT_OTHER_P.at(m).at(0).getServiceID() == oneSectionEit.getServiceID())
		{
			if (tables->EIT_OTHER_P.at(m).size() != 0)
			{
				int numberOfSectionInOneService = tables->EIT_OTHER_P.at(m).size();
				for (int n=0;n<numberOfSectionInOneService;n++)
				{
					if (tables->EIT_OTHER_P.at(m).size() == oneSectionEit.getLastSectionNumber()+1)
					{
						return;
					}
					if (tables->EIT_OTHER_P.at(m).at(n).getSectionNumber() != oneSectionEit.getSectionNumber())
					{
						tables->EIT_OTHER_P.at(m).push_back(oneSectionEit);
						if (tables->EIT_OTHER_P.at(m).size() == oneSectionEit.getLastSectionNumber()+1)
						{
							numberOfNotLoadedEitOtherPService--;
							if (numberOfNotLoadedEitOtherPService == 0)
							{
								eitOtherPLoaded = true;
							}
						}
					}
				}
				return;
			}
			else
			{
				tables->EIT_OTHER_P.at(m).push_back(oneSectionEit);
				if (tables->EIT_OTHER_P.at(m).size() == oneSectionEit.getLastSectionNumber()+1)
				{
					numberOfNotLoadedEitOtherPService--;
					if (numberOfNotLoadedEitOtherPService == 0)
					{
						eitOtherPLoaded = true;
						return;
					}
				}
			}
		}
	}

	oneProgramEitOther.push_back(oneSectionEit);
	tables->EIT_OTHER_P.push_back(oneProgramEitOther);
	if (tables->EIT_OTHER_P.back().size() == oneSectionEit.getLastSectionNumber()+1)
	{
		numberOfNotLoadedEitOtherPService--;
		if (numberOfNotLoadedEitOtherPService == 0)
		{
			eitOtherPLoaded = true;
		}
	}
}

//void DataLoader::completeEitOtherP(){} - zatim delano v locateAndSaveSectionEitOtherP

//eit actual schedule
void DataLoader::loadEitActualS()
{
	//eitActualSLoaded = true;	//toto zde vlozeno aby se tyto tabulky nehledaly
	if (eitActualSLoaded){return;}
	if (!patLoaded){return;}
	counterByte = 4+2;
	try
	{		
		loadEitBody();
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
		return;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
		return;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	locateAndSaveSectionEitActualS();
}

void DataLoader::locateAndSaveSectionEitActualS()
{
	//zde je naplnena oneSectionEit
	//ukolem tady je ji spravne zaradit

	vector<Eit> oneTableID;
	vector<vector<Eit>> oneProgramEitActual;

	if (tables->EIT_ACTUAL_S.size() == 0)
	{
		oneTableID.push_back(oneSectionEit);
		oneProgramEitActual.push_back(oneTableID);
		tables->EIT_ACTUAL_S.push_back(oneProgramEitActual);
		return;
	}

	int numberOfService = tables->EIT_ACTUAL_S.size();
	for (int m=0;m<numberOfService;m++)
	{
		//jestli je shoda serviceID
		if (oneSectionEit.getServiceID() == tables->EIT_ACTUAL_S.at(m).at(0).at(0).getServiceID())//je shoda serviceID)
		{
			int numberOfTable = tables->EIT_ACTUAL_S.at(m).size();
			for (int n=0;n<numberOfTable;n++)
			{
				//jestli je shoda tableID
				if (oneSectionEit.getTableId() == tables->EIT_ACTUAL_S.at(m).at(n).at(0).getTableId())//je shoda tableID)
				{
					int numberOfSection = tables->EIT_ACTUAL_S.at(m).at(n).size();
					int sameSection = 0;
					for (int j=0;j<numberOfSection;j++)
					{
						//jestli je shoda sectionNumb
						if (oneSectionEit.getSectionNumber() == tables->EIT_ACTUAL_S.at(m).at(n).at(j).getSectionNumber())
						{
							sameSection++;
						}
					}
					if (sameSection == 0)
					{
						//jestli je duplicitni section = 0 - je to ok
						//na aktualni tableID a aktualni service se ulozi section
						tables->EIT_ACTUAL_S.at(m).at(n).push_back(oneSectionEit);
						return;
					} 
					else
					{					
						//duplicitni sekce neni nula - sekce uz tam je
						//volani funkce completeEitSchedule();
						//pokud completeEitSchedule je true - eitActualSchedule = true
						if (completeEitActualS())
						{
							eitActualSLoaded = true;
						}
						return;
						//v aktualnim sercive a aktualni tableID zjisti jestli je plny
						//pokud je plny - vyskoci a zjisti jestli sou vsechny plne
					}
				} 
				else	//neni shoda tableID
				{
					//tak nic, projdou se vsechny table a jde se ven
				}
			}
			//vytvori se nova table
			oneTableID.push_back(oneSectionEit);
			//na aktualni service se tato tabulka ulozi
			tables->EIT_ACTUAL_S.at(m).push_back(oneTableID);
			return;
		}
		else //neni shoda serviceID
		{
			//tak nic, projdou se vsechny service a jde se ven
		}
	}

	//vytvori se nova tabuka
	oneTableID.push_back(oneSectionEit);
	//vytvori nova service
	oneProgramEitActual.push_back(oneTableID);
	//vyse uvedene se ulozi
	tables->EIT_ACTUAL_S.push_back(oneProgramEitActual);
}

bool DataLoader::completeEitActualS()
{
	if (numberOfPrograms == tables->EIT_ACTUAL_S.size())
	{
		for (unsigned int m=0;m<tables->EIT_ACTUAL_S.size();m++)
		{
			if (tables->EIT_ACTUAL_S.at(m).at(0).at(0).getLastTableID()+1-0x50 == tables->EIT_ACTUAL_S.at(m).size()) //pokud jsou kopletni table
			{
				for (unsigned int n=0;n<tables->EIT_ACTUAL_S.at(m).size();n++)
				{
					if (tables->EIT_ACTUAL_S.at(m).at(n).size() < (unsigned int) tables->EIT_ACTUAL_S.at(m).at(n).at(0).getLastSectionNumber()/8+1)	//pokud jsou koplentni section
					{
						return false;
					}
				}
			}
			else{return false;}
		}
	}
	else{return false;}
	return true;
}

//eit other schedule
void DataLoader::loadEitOtherS()
{
	//eitOtherSLoaded = true;
	if (eitOtherSLoaded){return;}
	if (!sdtOtherLoaded){return;}
	counterByte = 4+2;
	try
	{		
		loadEitBody();
	}
	catch (ScrambledException scramex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << scramex.what() << endl;
		cout << "Find scrambled TS packet" << endl;
		return;
	}
	catch (AdaptionFieldException afex)
	{
		//pokud tohle nastane musi vsechny tabulky ktere nejsou uplne vymazat
		cerr << afex.what() << endl;
		cout << "Find TS packet with adaption field" << endl;
		return;
	}
	catch(SynchronizationException syncex)
	{
		cerr << syncex.what() << endl;
		cout << "Synchronized exception. Return from loadDataFunction" << endl;
		return;
	}
	catch(TooLongTableException tLTex)
	{
		cerr << tLTex.what() << endl;
		return;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	catch(LostTsPacketException lTPex)
	{
		//cerr << lTPex.what() << endl;
		return;
	}
	locateAndSaveSectionEitOtherS();
}

void DataLoader::locateAndSaveSectionEitOtherS()
{
	vector<Eit> oneTableID;
	vector<vector<Eit>> oneProgramEitActual;
	//veskere popisy jsou k dispozici v obdobne funkci locateAndSaveSectionEitActualS
	if (tables->EIT_OTHER_S.size() == 0)
	{
		oneTableID.push_back(oneSectionEit);
		oneProgramEitActual.push_back(oneTableID);
		tables->EIT_OTHER_S.push_back(oneProgramEitActual);
		return;
	}
	int numberOfService = tables->EIT_OTHER_S.size();
	for (int m=0;m<numberOfService;m++)
	{
		//jestli je shoda serviceID
		if (oneSectionEit.getServiceID() == tables->EIT_OTHER_S.at(m).at(0).at(0).getServiceID())
		{
			int numberOfTable = tables->EIT_OTHER_S.at(m).size();
			for (int n=0;n<numberOfTable;n++)
			{
				//jestli je shoda tableID
				if (oneSectionEit.getTableId() == tables->EIT_OTHER_S.at(m).at(n).at(0).getTableId())
				{
					int numberOfSection = tables->EIT_OTHER_S.at(m).at(n).size();
					int sameSection = 0;
					for (int j=0;j<numberOfSection;j++)
					{
						//jestli je shoda sectionNumb
						if (oneSectionEit.getSectionNumber() == tables->EIT_OTHER_S.at(m).at(n).at(j).getSectionNumber())
						{
							sameSection++;
						}
					}
					if (sameSection == 0)
					{
						tables->EIT_OTHER_S.at(m).at(n).push_back(oneSectionEit);
						return;
					} 
					else
					{
						if (completeEitOtherS()){eitOtherSLoaded = true;}
						return;
					}
				} 
			}
			oneTableID.push_back(oneSectionEit);
			tables->EIT_OTHER_S.at(m).push_back(oneTableID);
			return;
		}
	}
	oneTableID.push_back(oneSectionEit);
	oneProgramEitActual.push_back(oneTableID);
	tables->EIT_OTHER_S.push_back(oneProgramEitActual);
}

bool DataLoader::completeEitOtherS()
{
	if (numberOfPrograms == tables->EIT_OTHER_S.size())
	{
		for (unsigned int m=0;m<tables->EIT_OTHER_S.size();m++)
		{
			if (tables->EIT_OTHER_S.at(m).at(0).at(0).getLastTableID()+1-0x50 == tables->EIT_OTHER_S.at(m).size()) //pokud jsou kopletni table
			{
				for (unsigned int n=0;n<tables->EIT_OTHER_S.at(m).size();n++)
				{
					if (tables->EIT_OTHER_S.at(m).at(n).size() < (unsigned int) tables->EIT_OTHER_S.at(m).at(n).at(0).getLastSectionNumber()/8+1)	//pokud jsou koplentni section
					{
						return false;
					}
				}
			}
			else{return false;}
		}
	}
	else{return false;}
	return true;
}

/******************************************************RST TABLE*******************************************************/
void DataLoader::loadRst()
{
	if (rstLoaded){return;}

	//Rst rst;

	//counterByte = 4+2;
	//RstEventLoop oneRstEventLoop;
	//vector<RstEventLoop> rstEventLoopOneRst;

	//rst.setTableID(tableID);
	//try
	//{
	//	int byteBuff = readByte();
	//	counterByte++;
	//	rst.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
	//	if (rst.getSectionSyntaxIndicator())
	//	{
	//		throw BadInformationAboutLengthDataException("Bad information about section syntax idicator when loading RST");
	//	}
	//	rst.setPrivateFlag(binaryShift(byteBuff,2,2));
	//	rst.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
	//	counterByte++;
	//	numberOfNotLoadedByte = rst.getSectionLength();	//nekonci CRC takze neni -4
	//	while (numberOfNotLoadedByte != 0)
	//	{
	//		//trasport stream ID
	//		byteBuff = readByte();
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		oneRstEventLoop.setTransportStreamID(byteBuff*0x100+readByte());
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		//original network ID
	//		byteBuff = readByte();
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		oneRstEventLoop.setOriginalNetworkID(byteBuff*0x100+readByte());
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		//service ID
	//		byteBuff = readByte();
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		oneRstEventLoop.setServiceID(byteBuff*0x100+readByte());
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		//event ID
	//		byteBuff = readByte();
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		oneRstEventLoop.setEventID(byteBuff*0x100+readByte());
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		//running status
	//		oneRstEventLoop.setRunningStatus(binaryShift(readByte(),6,8));
	//		counterByte++;
	//		numberOfNotLoadedByte--;
	//		questionAtEndTsPacket();
	//		//predavani smycky
	//		rstEventLoopOneRst.push_back(oneRstEventLoop);
	//		
	//		if (numberOfNotLoadedByte < 0)
	//		{
	//			throw BadInformationAboutLengthDataException("Bad information about length data when loading RST");
	//		}
	//	}
	//}
	//catch(BadInformationAboutLengthDataException bIALDex)
	//{
	//	cerr << bIALDex.what() << endl;
	//	return;
	//}
	//rst.setRstEventLoops(rstEventLoopOneRst);	//predani vektoru se vsemi smyckami jednoho programu do TableContaineru - vector<StreamLoops>
	///*for (int m=0;m<tables->RST.size();m++)
	//{
	//	RstEventLoop xxx = tables->RST.at(m).getRstEventLoops(rstEventLoops.at(0));
	//	RstEventLoop yyy = rst.getRstEventLoops(rstEventLoops.at(0));
	//	if (xxx.getServiceID() != yyy.getServiceID())
	//	{*/
	//		tables->RST.push_back(rst);					//predani rst do TableContaineru
	///*	}
	//}*/
	//
	//if (numberOfPrograms == tables->RST.size())
	//{
	//	rstLoaded = true;
	//}
	//tables->RST.setRstEventLoops(rstEventLoops);	//predani vektoru se vsemi smyckami do TableContaineru - vector<rstEventLoops>
	rstLoaded = true;
}

/******************************************************TDT TABLE*******************************************************/
void DataLoader::loadTdt()
{
	if (tdtLoaded){return;}

	counterByte = 4+2;
	tables->TDT.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->TDT.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->TDT.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->TDT.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;

		if (tables->TDT.getSectionSyntaxIndicator())
		{
			throw BadInformationAboutLengthDataException("Bad information about section syntax idicator when loading TDT");
		}
		if (tables->TDT.getSectionLength() != 5)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading TDT");
		}
		//MJD number
		int MJD = readByte()*0x100+readByte();
		counterByte = counterByte+2;
		//day
		int Yi =  int((MJD-15078.2)/365.25);
		int Mi =  int((MJD-14956.1-int(Yi*365.25))/30.6001);
		tables->TDT.setDay(MJD-14956-int(Yi*365.25)-int(Mi*30.6001));
		//month
		int K=0;
		if((Mi == 14)||(Mi == 15)){K=1;}
		tables->TDT.setMonth(Mi-1-K*12);
		//year
		int y;
		if((Yi+K-100) >= 0){y = 2000 + (Yi+K-100);}
		else{y = 1900 + (Yi+K-100);}
		tables->TDT.setYear(y);
		//day of week
		tables->TDT.setDayOfWeek(((MJD+2)%7)+1);
		//hour
		tables->TDT.setHour(readByte());
		counterByte++;
		//minute
		tables->TDT.setMinute(readByte());
		counterByte++;
		//second
		tables->TDT.setSecond(readByte());
		counterByte++;
	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	tdtLoaded = true;
}

/******************************************************TOT TABLE*******************************************************/
void DataLoader::loadTot()
{
	if (totLoaded){return;}

	counterByte = 4+2;
	tables->TOT.setTableID(tableID);
	try
	{
		int byteBuff = readByte();
		counterByte++;
		tables->TOT.setSectionSyntaxIndicator(binaryShift(byteBuff,1,1));
		tables->TOT.setPrivateFlag(binaryShift(byteBuff,2,2));
		tables->TOT.setSectionLength((binaryShift(byteBuff,5,8))*0x100+readByte());
		counterByte++;
		numberOfNotLoadedByte = tables->TOT.getSectionLength()-4;

		if (tables->TOT.getSectionSyntaxIndicator())
		{
			throw BadInformationAboutLengthDataException("Bad information about section syntax idicator when loading TDT");
		}
		
		//MJD number
		int MJD = readByte()*0x100+readByte();
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;
		//day
		int Yi =  int((MJD-15078.2)/365.25);
		int Mi =  int((MJD-14956.1-int(Yi*365.25))/30.6001);
		tables->TOT.setDay(MJD-14956-int(Yi*365.25)-int(Mi*30.6001));
		//month
		int K=0;
		if((Mi == 14)||(Mi == 15)){K=1;}
		tables->TOT.setMonth(Mi-1-K*12);
		//year
		int y;
		if((Yi+K-100) >= 0){y = 2000 + (Yi+K-100);}
		else{y = 1900 + (Yi+K-100);}
		tables->TOT.setYear(y);
		//day of week
		tables->TOT.setDayOfWeek(((MJD+2)%7)+1);
		//hour
		tables->TOT.setHour(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		//minute
		tables->TOT.setMinute(readByte());
		counterByte++;
		numberOfNotLoadedByte--;
		//second
		tables->TOT.setSecond(readByte());
		counterByte++;
		numberOfNotLoadedByte--;

		//tot descriptors length (infoLength)
		infoLength = binaryShift(readByte(),5,8)*0x100+readByte();
		counterByte = counterByte+2;
		numberOfNotLoadedByte = numberOfNotLoadedByte-2;

		//tot descriptors
		while (infoLength != 0)
		{
			descriptorIdentity(false);
		}
		if (numberOfNotLoadedByte < 0)
		{
			throw BadInformationAboutLengthDataException("Bad information about length data when loading TOT");
		}
		tables->TOT.setTotDescriptors(tableDescriptors);
		tableDescriptors.resize(0);

	}
	catch(BadInformationAboutLengthDataException bIALDex)
	{
		cerr << bIALDex.what() << endl;
		return;
	}
	totLoaded = true;
}
