#include "stdafx.h"
#include "DataLoader.h"
#include "TableContainer.h"

void DataLoader::siTableIdentity()
{
	continuityCounter();	//cte posledni byte hlavicky TS, vraci hodnotu do continuityCount
	tableID = readTableID();
	switch(PID)
	{
	case 0x00:
		if (tableID != TN_PAT)
		{
			throw BadTableIDException("Table PAT has incorrect table ID");
		}
		loadPat();
		numberOfPat++;
		break;

	case 0x01:
		if (tableID != TN_CAT)
		{
			throw BadTableIDException("Table CAT has incorrect table ID");
		}
		numberOfCat++;
		break;

	case 0x02:
		if (tableID != TN_TSDT)
		{
			throw BadTableIDException("Table TSDT has incorrect table ID");
		}
		numberOfTsdt++;
		break;

	case 0x10:
		switch(tableID)
		{
		case TN_NIT_ACTUAL:
			loadNitActual();
			numberOfNitActual++;
			break;
		case TN_NIT_OTHER:
			loadNitOther();
			numberOfNitOther++;
			break;
		case TN_ST:
			numberOfSt++;
			break;
		default:
			throw BadTableIDException("Incorrect table ID in PID 0x10");
		}
		break;

	case 0x11:
		switch(tableID)
		{
		case TN_SDT_ACTUAL:
			loadSdtActual();
			numberOfSdtActual++;
			break;
		case TN_SDT_OTHER:
			loadSdtOther();
			numberOfSdtOther++;
			break;
		case TN_BAT:
			loadBat();
			numberOfBat++;
			break;
		case TN_ST:
			numberOfSt++;
			break;
		default:
			throw BadTableIDException("Incorrect table ID in PID 0x11");
		}
		break;

	case 0x12:
		switch(tableID)
		{
		case TN_EIT_ACTUAL_P:
			loadEitActualP();
			numberOfEitActualP++;
			break;
		case TN_EIT_OTHER_P:
			loadEitOtherP();
			numberOfEitOtherP++;
			break;
		case TN_ST:
			numberOfSt++;
			break;
		default:
			if (tableID >= 0x50 || tableID <= 0x5F)
			{
				loadEitActualS();
				numberOfEitActualS++;
			}
			else if (tableID >= 0x60 || tableID <= 0x6F)
			{
				loadEitOtherS();
				numberOfEitOtherS++;
			}
			else
			{
				throw BadTableIDException("Incorrect table ID in PID 0x12");
			}
		}
		break;

	case 0x13:
		switch(tableID)
		{
		case TN_RST:
			loadRst();
			numberOfRst++;
			break;
		case TN_ST:
			numberOfSt++;
		default:
			throw BadTableIDException("Incorrect table ID in PID 0x13");
		}
		break;

	case 0x14:
		switch(tableID)
		{
		case TN_TDT:
			loadTdt();
			numberOfTdt++;
			break;
		case TN_TOT:
			loadTot();
			numberOfTot++;
			break;
		case TN_ST:
			numberOfSt++;
			break;
		default:
			throw BadTableIDException("Incorrect table ID in PID 0x14");
		}
		break;

	case 0x15:
		numberOfSyncNetwork++;
		break;

	case 0x16:
		numberOfRnt++;
		break;

	case 0x1C:
		numberOfMetaData++;
		break;

	case 0x1D:
		numberOfMeasurement++;
		break;

	case 0x1E:
		numberOfDit++;
		break;

	case 0x1F:
		numberOfSit++;
		break;

	default:
		cout << "Unknown PID" << endl;
	}
}

void DataLoader::pmtsTableIdentity()
{
	if (!patLoaded)
	{
		return;
	}
	
	for (unsigned int m=0; m<programNumbers.size(); m++)
	{
		if (pmtPIDs.at(m) == PID)
		{
			if (programNumbers.at(m) != 0)
			{
				continuityCounter();	//cte posledni byte hlavicky TS, vraci hodnotu do continuityCount
				tableID = readTableID();
				if (tableID != TN_PMT)
				{
					throw BadTableIDException("Table PMT has incorrect table ID");
				}
				loadPmt();							//pozor toto se bude volat tolikrat kolik bude programu
				numberOfPmts++;
			}
		}
	}
}
